#!/bin/bash
#
# Firewall pro malou firmu
# Zpracoval Josef Vitek podle skriptu Miroslava Petricka (http://www.petricek.cz/mpfw/mpfw)

# Ochrana pred spoofingem
if [ -e /proc/sys/net/ipv4/conf/all/rp_filter ]
then
for filtre in /proc/sys/net/ipv4/conf/*/rp_filter
do
echo 1 > $filtre
done
fi 

# Nacteni uzitecnych modulu
modprobe ip_tables
modprobe ip_nat_ftp
modprobe ip_nat_irc
modprobe iptable_filter
modprobe iptable_nat 
modprobe ip_conntrack_irc
modprobe ip_conntrack_ftp

# Forwarding
echo "1" > /proc/sys/net/ipv4/ip_forward
echo "1" > /proc/sys/net/ipv4/tcp_syncookies

echo " [Mazu stara pravidla iptables]" 
# Vymazani pravidel a retezcu
iptables -F
iptables -X

echo " [Nastavuji iptables]" 
# Nastaveni vychozi politiky
iptables -P INPUT DROP
iptables -P OUTPUT DROP
iptables -P FORWARD DROP

# Retezec spoofing (zahazujeme podvrzene pakety na vnejsim rozhrani)
iptables -N spoofing
iptables -A spoofing -s 192.168.0.0/16 -j DROP
iptables -A spoofing -s 172.16.0.0/12 -j DROP
iptables -A spoofing -s 10.0.0.0/8 -j DROP


# IP maskarada (NAT)
iptables -t nat -A POSTROUTING -o eth1 -j SNAT --to 192.168.1.1


# Retezec FORWARD

# Navazovani spojeni ala Microsoft -
# Paket navazuje spojeni, ale nema nastaveny priznak SYN, pryc s nim
iptables -A FORWARD -p tcp ! --syn -m state --state NEW -j DROP

iptables -A FORWARD -p tcp -i eth1 --tcp-flags SYN,FIN SYN,FIN -j LOG -m limit --limit 10/m --log-prefix="bogus packet: "
iptables -A FORWARD -p tcp -i eth1 --tcp-flags SYN,FIN SYN,FIN -j DROP

# Routing zevnitr site ven omezujeme na urcite sluzby + vyjimky

iptables -A FORWARD -i eth0 -j DROP
iptables -A FORWARD -i eth1 -j spoofing

iptables -A FORWARD -i eth0 -p TCP --dport 21 -j ACCEPT  #FTP 
iptables -A FORWARD -i eth0 -p TCP --dport 22 -j ACCEPT  #SSH 
iptables -A FORWARD -i eth0 -p TCP --dport 25 -j ACCEPT  #SMTP 
iptables -A FORWARD -i eth0 -p UDP --dport 53 -j ACCEPT  #DNS  UDP
iptables -A FORWARD -i eth0 -p TCP --dport 53 -j ACCEPT  #DNS  TCP
iptables -A FORWARD -i eth0 -p TCP --dport 80 -j ACCEPT  #WWW 
iptables -A FORWARD -i eth0 -p TCP --dport 110 -j ACCEPT #POP3 
iptables -A FORWARD -i eth0 -p TCP --dport 143 -j ACCEPT #IMAP 
iptables -A FORWARD -i eth0 -p TCP --dport 443 -j ACCEPT #HTTPS 
iptables -A FORWARD -i eth0 -p TCP --dport 873 -j ACCEPT #rsync 

iptables -A FORWARD -i eth0 -s 192.168.0.2 -j ACCEPT
iptables -A FORWARD -i eth0 -s 192.168.0.10 -j ACCEPT
iptables -A FORWARD -i eth0 -s 192.168.0.11 -j ACCEPT
iptables -A FORWARD -i eth0 -s 192.168.0.20 -j ACCEPT

# Routing zvenku dovnitr pouze pro navazana spojeni (stavovy firewall)
iptables -A FORWARD -i eth1 -o eth0 -m state --state ESTABLISHED,RELATED -j ACCEPT

# Ostatni pakety budou zahozeny, tak je budeme logovat (12 x 5 pkt/hod)
iptables -A FORWARD -m limit --limit 12/h -j LOG --log-prefix "forward drop:"

# Retezec INPUT

iptables -A INPUT -i eth1 -j spoofing

# Navazovani spojeni ala Microsoft -
# Paket navazuje spojeni, ale nema nastaveny priznak SYN, pryc s nim

iptables -A INPUT -p tcp ! --syn -m state --state NEW -j DROP

# Ochrana pred SYN floodem (pet paketu/s SYN je povoleno)
iptables -N syn_flood
iptables -A INPUT -i eth0 -p tcp --syn -j syn_flood
iptables -A syn_flood -m limit --limit 1/s --limit-burst 5 -j RETURN
iptables -A syn_flood -j DROP


# Portscan s nastavenym SYN,FIN
iptables -A INPUT -p tcp -i eth1 --tcp-flags SYN,FIN SYN,FIN -j LOG -m limit --limit 10/m --log-prefix="bogus packet: "
iptables -A INPUT -p tcp -i eth1 --tcp-flags SYN,FIN SYN,FIN -j DROP


# Sluzbu AUTH neni dobre filtrovat pomoci DROP, protoze to muz
# vest k prodlevam pri navazovani nekterych spojeni. Proto jej
# sice zamitneme, ale tak, aby nedoslo k nezadoucim prodlevam.
iptables -A INPUT -i eth1 -p TCP --dport 113 -m limit --limit 12/h -j LOG
iptables -A INPUT -i eth1 -p TCP --dport 113 -j REJECT --reject-with tcp-reset #AUTH server

# Propoustime pouze ICMP ping
iptables -A INPUT -i eth1 -p ICMP --icmp-type echo-request -j ACCEPT

# Loopback neomezovat
iptables -A INPUT -i lo -j ACCEPT

# Pakety z lokalni site smerovane na firewall povolit
iptables -A INPUT -i eth0 -d 192.168.0.1 -j ACCEPT
iptables -A INPUT -i eth0 -d 192.168.1.1 -j ACCEPT

# Broadcasty na lokalni siti povolit
iptables -A INPUT -i eth0 -d 192.168.0.255 -j ACCEPT

# Chyba v implementaci DHCP
iptables -A INPUT -i eth0 -p udp --dport 67 -j ACCEPT

# Stavovy firewall, pakety od navazanych spojeni povolime
iptables -A INPUT -d 192.168.1.1 -m state --state ESTABLISHED,RELATED -j ACCEPT

# Vsechno ostatni je zakazano - tedy logujeme, maxim. 12x5 pkt/hod 
iptables -A INPUT -m limit --limit 12/h -j LOG --log-prefix "INPUT drop: "

# Retezec OUTPUT

# Povolime odchozi pakety, ktere maji nase IP adresy
iptables -A OUTPUT -s 127.0.0.1 -j ACCEPT
iptables -A OUTPUT -s 192.168.0.1 -j ACCEPT
iptables -A OUTPUT -s 192.168.1.1 -j ACCEPT

# DHCP broadcasty na lokalni sit povolit
iptables -A OUTPUT -o eth0 -p UDP --dport 68 --sport 67 -j ACCEPT

# Ostatni pakety logujeme (nemely by byt zadne takove)
iptables -A OUTPUT -j LOG --log-prefix "OUTPUT drop: "


echo " [Iptables nastaveno]" 
